@@ -28,6 +28,8 @@
 
 enum tca_flags {tca_on, tca_off};
 enum rsa_flags {rsa_off, rsa_on};
+enum tca_idm_dr_flags {tca_idm_dr_on, tca_idm_dr_off};
+enum rsa_idr_flags {rsa_idr_off, rsa_idr_on};
 enum ufa_flags {ufa_off, ufa_on};
 enum ncdmfa_flags {ncdmfa_off, ncdmfa_on};
 
@@ -41,6 +43,8 @@ enum ncdmfa_flags {ncdmfa_off, ncdmfa_on};
 
 enum tca_method {first_order_MB,first_order_CAMB,first_order_CLASS,second_order_CRS,second_order_CLASS,compromise_CLASS};
 enum rsa_method {rsa_null,rsa_MD,rsa_MD_with_reio,rsa_none};
+enum idr_method {idr_free_streaming,idr_fluid}; /* for the idm-idr case */
+enum rsa_idr_method {rsa_idr_none,rsa_idr_MD};  /* for the idm-idr case */
 enum ufa_method {ufa_mb,ufa_hu,ufa_CLASS,ufa_none};
 enum ncdmfa_method {ncdmfa_mb,ncdmfa_hu,ncdmfa_CLASS,ncdmfa_none};
 enum tensor_methods {tm_photons_only,tm_massless_approximation,tm_exact};
@@ -170,6 +174,11 @@ struct perturbs
 
   double z_max_pk; /**< when we compute only the matter spectrum / transfer functions, but not the CMB, we are sometimes interested to sample source functions at very high redshift, way before recombination. This z_max_pk will then fix the initial sampling time of the sources. */
 
+  double * alpha_idm_dr; /**< Angular contribution to collisional term at l>=2 for idm_fr-idr */
+  double * beta_idr;  /**< Angular contribution to collisional term at l>=2 for idr-idr */
+
+  int idr_nature; /**< Nature of the interacting dark radiation (free streaming or fluid) */
+
   //@}
 
   /** @name - useful flags inferred from the ones above */
@@ -220,42 +229,46 @@ struct perturbs
 
   //@{
 
-  short has_source_t;  /**< do we need source for CMB temperature? */
-  short has_source_p;  /**< do we need source for CMB polarization? */
-  short has_source_delta_m;   /**< do we need source for delta of total matter? */
-  short has_source_delta_cb; /**< do we ALSO need source for delta of ONLY cdm and baryon? */
-  short has_source_delta_tot;  /**< do we need source for delta total? */
-  short has_source_delta_g;    /**< do we need source for delta of gammas? */
-  short has_source_delta_b;    /**< do we need source for delta of baryons? */
-  short has_source_delta_cdm;  /**< do we need source for delta of cold dark matter? */
-  short has_source_delta_dcdm; /**< do we need source for delta of DCDM? */
-  short has_source_delta_fld;  /**< do we need source for delta of dark energy? */
-  short has_source_delta_scf;  /**< do we need source for delta from scalar field? */
-  short has_source_delta_dr; /**< do we need source for delta of decay radiation? */
-  short has_source_delta_ur; /**< do we need source for delta of ultra-relativistic neutrinos/relics? */
-  short has_source_delta_ncdm; /**< do we need source for delta of all non-cold dark matter species (e.g. massive neutrinos)? */
-  short has_source_theta_m;    /**< do we need source for theta of total matter? */
-  short has_source_theta_cb; /**< do we ALSO need source for theta of ONLY cdm and baryon? */
-  short has_source_theta_tot;  /**< do we need source for theta total? */
-  short has_source_theta_g;    /**< do we need source for theta of gammas? */
-  short has_source_theta_b;    /**< do we need source for theta of baryons? */
-  short has_source_theta_cdm;  /**< do we need source for theta of cold dark matter? */
-  short has_source_theta_dcdm; /**< do we need source for theta of DCDM? */
-  short has_source_theta_fld;  /**< do we need source for theta of dark energy? */
-  short has_source_theta_scf;  /**< do we need source for theta of scalar field? */
-  short has_source_theta_dr; /**< do we need source for theta of ultra-relativistic neutrinos/relics? */
-  short has_source_theta_ur; /**< do we need source for theta of ultra-relativistic neutrinos/relics? */
-  short has_source_theta_ncdm; /**< do we need source for theta of all non-cold dark matter species (e.g. massive neutrinos)? */
-  short has_source_phi;          /**< do we need source for metric fluctuation phi? */
-  short has_source_phi_prime;    /**< do we need source for metric fluctuation phi'? */
-  short has_source_phi_plus_psi; /**< do we need source for metric fluctuation (phi+psi)? */
-  short has_source_psi;          /**< do we need source for metric fluctuation psi? */
-  short has_source_h;            /**< do we need source for metric fluctuation h? */
-  short has_source_h_prime;      /**< do we need source for metric fluctuation h'? */
-  short has_source_eta;          /**< do we need source for metric fluctuation eta? */
-  short has_source_eta_prime;    /**< do we need source for metric fluctuation eta'? */
-  short has_source_H_T_Nb_prime;    /**< do we need source for metric fluctuation H_T_Nb'? */
-  short has_source_k2gamma_Nb;    /**< do we need source for metric fluctuation gamma in Nbody gauge? */
+  short has_source_t;           /**< do we need source for CMB temperature? */
+  short has_source_p;           /**< do we need source for CMB polarization? */
+  short has_source_delta_m;     /**< do we need source for delta of total matter? */
+  short has_source_delta_cb;    /**< do we ALSO need source for delta of ONLY cdm and baryon? */
+  short has_source_delta_tot;   /**< do we need source for delta total? */
+  short has_source_delta_g;     /**< do we need source for delta of gammas? */
+  short has_source_delta_b;     /**< do we need source for delta of baryons? */
+  short has_source_delta_cdm;   /**< do we need source for delta of cold dark matter? */
+  short has_source_delta_dcdm;  /**< do we need source for delta of DCDM? */
+  short has_source_delta_fld;   /**< do we need source for delta of dark energy? */
+  short has_source_delta_scf;   /**< do we need source for delta from scalar field? */
+  short has_source_delta_dr;    /**< do we need source for delta of decay radiation? */
+  short has_source_delta_ur;    /**< do we need source for delta of ultra-relativistic neutrinos/relics? */
+  short has_source_delta_idr;   /**< do we need source for delta of interacting dark radiation? */
+  short has_source_delta_idm_dr;/**< do we need source for delta of interacting dark matter (with dr)? */
+  short has_source_delta_ncdm;  /**< do we need source for delta of all non-cold dark matter species (e.g. massive neutrinos)? */
+  short has_source_theta_m;     /**< do we need source for theta of total matter? */
+  short has_source_theta_cb;    /**< do we ALSO need source for theta of ONLY cdm and baryon? */
+  short has_source_theta_tot;   /**< do we need source for theta total? */
+  short has_source_theta_g;     /**< do we need source for theta of gammas? */
+  short has_source_theta_b;     /**< do we need source for theta of baryons? */
+  short has_source_theta_cdm;   /**< do we need source for theta of cold dark matter? */
+  short has_source_theta_dcdm;  /**< do we need source for theta of DCDM? */
+  short has_source_theta_fld;   /**< do we need source for theta of dark energy? */
+  short has_source_theta_scf;   /**< do we need source for theta of scalar field? */
+  short has_source_theta_dr;    /**< do we need source for theta of ultra-relativistic neutrinos/relics? */
+  short has_source_theta_ur;    /**< do we need source for theta of ultra-relativistic neutrinos/relics? */
+  short has_source_theta_idr;   /**< do we need source for theta of interacting dark radiation? */
+  short has_source_theta_idm_dr;/**< do we need source for theta of interacting dark matter (with dr)? */
+  short has_source_theta_ncdm;  /**< do we need source for theta of all non-cold dark matter species (e.g. massive neutrinos)? */
+  short has_source_phi;         /**< do we need source for metric fluctuation phi? */
+  short has_source_phi_prime;   /**< do we need source for metric fluctuation phi'? */
+  short has_source_phi_plus_psi;/**< do we need source for metric fluctuation (phi+psi)? */
+  short has_source_psi;         /**< do we need source for metric fluctuation psi? */
+  short has_source_h;           /**< do we need source for metric fluctuation h? */
+  short has_source_h_prime;     /**< do we need source for metric fluctuation h'? */
+  short has_source_eta;         /**< do we need source for metric fluctuation eta? */
+  short has_source_eta_prime;   /**< do we need source for metric fluctuation eta'? */
+  short has_source_H_T_Nb_prime;/**< do we need source for metric fluctuation H_T_Nb'? */
+  short has_source_k2gamma_Nb;  /**< do we need source for metric fluctuation gamma in Nbody gauge? */
 
   /* remember that the temperature source function includes three
      terms that we call 0,1,2 (since the strategy in class v > 1.7 is
@@ -276,22 +289,26 @@ struct perturbs
   int index_tp_delta_scf;  /**< index value for delta of scalar field */
   int index_tp_delta_dr; /**< index value for delta of decay radiation */
   int index_tp_delta_ur; /**< index value for delta of ultra-relativistic neutrinos/relics */
+  int index_tp_delta_idr; /**< index value for delta of interacting dark radiation */
+  int index_tp_delta_idm_dr;/**< index value for delta of interacting dark matter (with dr)*/
   int index_tp_delta_ncdm1; /**< index value for delta of first non-cold dark matter species (e.g. massive neutrinos) */
   int index_tp_perturbed_recombination_delta_temp;		/**< Gas temperature perturbation */
   int index_tp_perturbed_recombination_delta_chi;		/**< Inionization fraction perturbation */
 
-  int index_tp_theta_m;    /**< index value for matter velocity fluctuation */
-  int index_tp_theta_cb;   /**< index value for theta cb */
-  int index_tp_theta_tot;  /**< index value for total velocity fluctuation */
-  int index_tp_theta_g;    /**< index value for theta of gammas */
-  int index_tp_theta_b;    /**< index value for theta of baryons */
-  int index_tp_theta_cdm;  /**< index value for theta of cold dark matter */
-  int index_tp_theta_dcdm; /**< index value for theta of DCDM */
-  int index_tp_theta_fld;  /**< index value for theta of dark energy */
-  int index_tp_theta_scf;  /**< index value for theta of scalar field */
-  int index_tp_theta_ur;   /**< index value for theta of ultra-relativistic neutrinos/relics */
-  int index_tp_theta_dr;   /**< index value for F1 of decay radiation */
-  int index_tp_theta_ncdm1;/**< index value for theta of first non-cold dark matter species (e.g. massive neutrinos) */
+  int index_tp_theta_m;     /**< index value for matter velocity fluctuation */
+  int index_tp_theta_cb;    /**< index value for theta cb */
+  int index_tp_theta_tot;   /**< index value for total velocity fluctuation */
+  int index_tp_theta_g;     /**< index value for theta of gammas */
+  int index_tp_theta_b;     /**< index value for theta of baryons */
+  int index_tp_theta_cdm;   /**< index value for theta of cold dark matter */
+  int index_tp_theta_dcdm;  /**< index value for theta of DCDM */
+  int index_tp_theta_fld;   /**< index value for theta of dark energy */
+  int index_tp_theta_scf;   /**< index value for theta of scalar field */
+  int index_tp_theta_ur;    /**< index value for theta of ultra-relativistic neutrinos/relics */
+  int index_tp_theta_idr;   /**< index value for theta of interacting dark radiation */
+  int index_tp_theta_idm_dr;/**< index value for theta of interacting dark matter (with dr)*/
+  int index_tp_theta_dr;    /**< index value for F1 of decay radiation */
+  int index_tp_theta_ncdm1; /**< index value for theta of first non-cold dark matter species (e.g. massive neutrinos) */
 
   int index_tp_phi;          /**< index value for metric fluctuation phi */
   int index_tp_phi_prime;    /**< index value for metric fluctuation phi' */
@@ -302,7 +319,7 @@ struct perturbs
   int index_tp_eta;          /**< index value for metric fluctuation eta */
   int index_tp_eta_prime;    /**< index value for metric fluctuation eta' */
   int index_tp_H_T_Nb_prime; /**< index value for metric fluctuation H_T_Nb' */
-  int index_tp_k2gamma_Nb;     /**< index value for metric fluctuation gamma times k^2 in Nbody gauge */
+  int index_tp_k2gamma_Nb;   /**< index value for metric fluctuation gamma times k^2 in Nbody gauge */
 
   int * tp_size; /**< number of types tp_size[index_md] included in computation for each mode */
 
@@ -444,6 +461,8 @@ struct perturb_vector
   int index_pt_theta_b;   /**< baryon velocity */
   int index_pt_delta_cdm; /**< cdm density */
   int index_pt_theta_cdm; /**< cdm velocity */
+  int index_pt_delta_idm_dr;/**< idm_dr density */
+  int index_pt_theta_idm_dr;/**< idm_dr velocity */
   int index_pt_delta_dcdm; /**< dcdm density */
   int index_pt_theta_dcdm; /**< dcdm velocity */
   int index_pt_delta_fld;  /**< dark energy density in true fluid case */
@@ -456,6 +475,12 @@ struct perturb_vector
   int index_pt_shear_ur; /**< shear of ultra-relativistic neutrinos/relics */
   int index_pt_l3_ur;    /**< l=3 of ultra-relativistic neutrinos/relics */
   int l_max_ur;          /**< max momentum in Boltzmann hierarchy (at least 3) */
+  int index_pt_delta_idr; /**< density of interacting dark radiation */
+  int index_pt_theta_idr; /**< velocity of interacting dark radiation */
+  int index_pt_shear_idr; /**< shear of interacting dark radiation */
+  int index_pt_l3_idr;    /**< l=3 of interacting dark radiation */
+  int l_max_idr;          /**< max momentum in Boltzmann hierarchy (at least 3) for interacting dark radiation */
+
 /* perturbed recombination */
   int index_pt_perturbed_recombination_delta_temp;		/**< Gas temperature perturbation */
   int index_pt_perturbed_recombination_delta_chi;		/**< Inionization fraction perturbation */
@@ -545,10 +570,13 @@ struct perturb_workspace
 
   double tca_shear_g;  /**< photon shear in tight-coupling approximation */
   double tca_slip;     /**< photon-baryon slip in tight-coupling approximation */
+  double tca_shear_idm_dr;/**< interacting dark radiation shear in tight coupling appproximation */
   double rsa_delta_g;  /**< photon density in radiation streaming approximation */
   double rsa_theta_g;  /**< photon velocity in radiation streaming approximation */
   double rsa_delta_ur; /**< photon density in radiation streaming approximation */
   double rsa_theta_ur; /**< photon velocity in radiation streaming approximation */
+  double rsa_delta_idr; /**< interacting dark radiation density in dark radiation streaming approximation */
+  double rsa_theta_idr; /**< interacting dark radiation velocity in dark radiation streaming approximation */
 
   double * delta_ncdm;	/**< relative density perturbation of each ncdm species */
   double * theta_ncdm;	/**< velocity divergence theta of each ncdm species */
@@ -588,6 +616,8 @@ struct perturb_workspace
 
   int index_ap_tca; /**< index for tight-coupling approximation */
   int index_ap_rsa; /**< index for radiation streaming approximation */
+  int index_ap_tca_idm_dr; /**< index for dark tight-coupling approximation (idm-idr) */
+  int index_ap_rsa_idr; /**< index for dark radiation streaming approximation */
   int index_ap_ufa; /**< index for ur fluid approximation */
   int index_ap_ncdmfa; /**< index for ncdm fluid approximation */
   int ap_size;      /**< number of relevant approximations for a given mode */
@@ -873,6 +903,18 @@ extern "C" {
                                   struct perturb_workspace * ppw
                                   );
 
+  int perturb_rsa_idr_delta_and_theta(
+                                  struct precision * ppr,
+                                  struct background * pba,
+                                  struct thermo * pth,
+                                  struct perturbs * ppt,
+                                  double k,
+                                  double * y,
+                                  double a_prime_over_a,
+                                  double * pvecthermo,
+                                  struct perturb_workspace * ppw
+                                  );
+
 #ifdef __cplusplus
 }
 #endif
